"""
Sims 4 Change Gender Console Command
by Scumbumbo @ ModTheSims
"""
import sims4.commands
import sims.sim_info
import services
import enum
from sims.sim_info_types import Gender
from sims.sim_outfits import OutfitCategory
import sims.sim_outfits
from sims.global_gender_preference_tuning import GlobalGenderPreferenceTuning

def sg_show_usage(output):
    output("usage: setgender NewGender [First Last]")

def sg_swap_gender_pref(info):
    tracker = info.get_tracker(GlobalGenderPreferenceTuning.GENDER_PREFERENCE[Gender.MALE])
    mpref = tracker.get_value(GlobalGenderPreferenceTuning.GENDER_PREFERENCE[Gender.MALE])
    fpref = tracker.get_value(GlobalGenderPreferenceTuning.GENDER_PREFERENCE[Gender.FEMALE])
    tracker.set_value(GlobalGenderPreferenceTuning.GENDER_PREFERENCE[Gender.MALE], fpref)
    tracker.set_value(GlobalGenderPreferenceTuning.GENDER_PREFERENCE[Gender.FEMALE], mpref)

@sims4.commands.Command('setgender', command_type=sims4.commands.CommandType.Live)
def change_sim_gender(new_gender=None, fname=None, lname=None, _connection=None):
    output = sims4.commands.CheatOutput(_connection)

    if new_gender is None or ( fname is not None and lname is None ):
        sg_show_usage(output)
        return False

    if fname is not None:
        info = services.sim_info_manager().get_sim_info_by_name(fname, lname)
        if info is None:
            output("Error: Sim {} {} not found".format(fname, lname))
            return False
    else:
        tgt_client = services.client_manager().get(_connection)
        if tgt_client is not None:
            info = tgt_client.active_sim.sim_info
        else:
            output("Error: no sim targetted")
            sg_show_usage(output)
            return False

    if new_gender == "m" or new_gender == "male":
        if info.is_male:
            output("{} {} is already male!".format(info.first_name, info.last_name))
            return False
        else:
            info.gender = Gender.MALE
            sg_swap_gender_pref(info)
            output("{} {} is now a male".format(info.first_name, info.last_name))
    elif new_gender == "f" or new_gender == "female":
        if info.is_female:
            output("{} {} is already female!".format(info.first_name, info.last_name))
            return False
        else:
            info.gender = Gender.FEMALE
            sg_swap_gender_pref(info)
            output("{} {} is now a female".format(info.first_name, info.last_name))
    else:
        sg_show_usage(output)
        return False

    # Old outfits are incompatible with the new sex, generate new outfits
    info.generate_outfit(OutfitCategory.EVERYDAY)
    info.generate_outfit(OutfitCategory.FORMAL)
    info.generate_outfit(OutfitCategory.ATHLETIC)
    info.generate_outfit(OutfitCategory.SLEEP)
    info.generate_outfit(OutfitCategory.PARTY)
    info.generate_outfit(OutfitCategory.BATHING)
    # Don't think there's any need to generate the following two outfits, they appear to be generated on the fly
    #info.generate_outfit(OutfitCategory.SITUATION)
    #info.generate_outfit(OutfitCategory.SPECIAL)
    info.generate_career_outfit()
    
    # Change into the new outfit, I do this twice in case the target is already wearing (EVERYDAY, 0)
    info.set_current_outfit((OutfitCategory.FORMAL, 0))
    info.set_current_outfit((OutfitCategory.EVERYDAY, 0))

    info.save_sim
